@@ -314,14 +314,12 @@ def test_horne_interpolated_profile(mk_test_img):
 
     assert_quantity_allclose(horne_extract_gauss.spectrum.flux, horne_extract_self.spectrum.flux)
 
-    with pytest.raises(ValueError, match="When"):
-        HorneExtract(
-            image.data,
-            trace,
-            spatial_profile={"name": "interpolated_profile", "n_bins_interpolated_profile": 3},
-            bkgrd_prof=models.Polynomial1D(2),
-            variance=np.ones(image.data.shape),
-        ).spectrum
+    horne_extract_self = HorneExtract(
+        image.data,
+        trace,
+        spatial_profile={"name": "interpolated_profile", "n_bins_interpolated_profile": 3},
+        variance=np.ones(image.data.shape),
+    )
 
 
 def test_horne_interpolated_profile_norm(mk_test_img):
